@@ -32,25 +32,6 @@ local function inlocale(fn)
   os.setlocale(oldloc, 'numeric')
 end
 
-if test_module == 'dkjson-nopeg' then
-  test_module = 'dkjson'
-  package.preload["lpeg"] = function () error "lpeg disabled" end
-  package.loaded["lpeg"] = nil
-  lpeg = nil
-end
-
-if test_module == 'dkjson-lulpeg' then
-  test_module = 'dkjson'
-  package.loaded["lpeg"] = require "lulpeg"
-end
-
-do
-  -- http://chiselapp.com/user/dhkolf/repository/dkjson/
-  local dkjson = require "dkjson"
-  dkencode = dkjson.encode
-  dkdecode = dkjson.decode
-end
-
 if test_module == 'cmj-json' then
   -- https://github.com/craigmj/json4lua/
   -- http://json.luaforge.net/
@@ -59,8 +40,20 @@ if test_module == 'cmj-json' then
   decode = json.decode
 elseif test_module == 'dkjson' then
   -- http://chiselapp.com/user/dhkolf/repository/dkjson/
-  encode = dkencode
-  decode = dkdecode
+  local json = require "dkjson"
+  encode = json.encode
+  decode = json.decode
+elseif test_module == 'dkjson-lpeg' then
+  test_module = 'dkjson'
+  local json = require "dkjson".use_lpeg()
+  encode = json.encode
+  decode = json.decode
+elseif test_module == 'dkjson-lulpeg' then
+  test_module = 'dkjson'
+  package.loaded["lpeg"] = require "lulpeg"
+  local json = require "dkjson".use_lpeg()
+  encode = json.encode
+  decode = json.decode
 elseif test_module == 'fleece' then
   -- http://www.eonblast.com/fleece/
   local fleece = require "fleece"
@@ -101,6 +94,13 @@ else
   return
 end
 
+do
+  -- http://chiselapp.com/user/dhkolf/repository/dkjson/
+  local dkjson = require "dkjson"
+  dkencode = dkjson.encode
+  dkdecode = dkjson.decode
+end
+
 if not encode then
   print ("No encode method")
 else
@@ -325,6 +325,13 @@ else
       if origstate.buffer ~= origbuffer then print("expected original buffer to remain") end
       if origstate.bufferlen ~= 1 then print("expected bufferlen to be updated") end
     end
+
+    do -- order keys when value is 'false'
+      local r = dkencode({a = false, b = true, c = true}, {keyorder = {"a", "b", "c"}})
+      if r ~= [[{"a":false,"b":true,"c":true}]] then
+        print("unexpected keyorder:", r)
+      end
+    end
   end
 end
 
@@ -333,6 +340,36 @@ if not decode then
 else
   local x, r
 
+  x = decode [=[
+{
+  "String": "Content",
+  "Integer": 26,
+  "Object": {
+    "String2": "Content2",
+    "Boolean": true,
+    "Object2": {},
+    "Array": [ "Aa", "Bb", "Cc" ],
+    "Integer": 42,
+    "Nil": null
+  }
+}
+]=]
+  r = type(x) == 'table'
+  r = r and x.String == "Content"
+  r = r and x.Integer == 26
+  r = r and type(x.Object) == 'table'
+  r = r and x.Object.String2 == "Content2"
+  r = r and x.Object.Boolean == true
+  r = r and type(x.Object.Object2) == 'table'
+  r = r and type(x.Object.Array) == 'table'
+  r = r and x.Object.Array[3] == "Cc"
+  r = r and x.Object.Integer == 42
+  r = r and x.Object.Nil == nil
+  if not r then
+    print ("Did not decode example data correctly")
+  end
+
+
   x = decode[=[ ["\u0000"] ]=]
   if x[1] ~= "\000" then
     print ("\\u0000 isn't decoded correctly")
@@ -390,11 +427,29 @@ else
   r, x = pcall(decode, "{}")
   if not r then
     print ("decoding an empty object raises an error:", x)
+  elseif type(x) ~= 'table' then
+    print ("decoding an empty object did not return a table")
+  end
+
+  r, x = pcall(decode, "{\"a\":1}")
+  if not r then
+    print ("decoding an object with one element raises an error:", x)
+  elseif type(x) ~= 'table' or x.a ~= 1 then
+    print ("decoding an object with one element did not return the expected table")
+  end
+
+  r, x = pcall(decode, "{\"a\":1,\"b\":2}")
+  if not r then
+    print ("decoding an object with two elements raises an error:", x)
+  elseif type(x) ~= 'table' or x.a ~= 1 or x.b ~= 2 then
+    print ("decoding an object with two elements did not return the expected table")
   end
 
   r, x = pcall(decode, "[]")
   if not r then
     print ("decoding an empty array raises an error:", x)
+  elseif type(x) ~= 'table' then
+    print ("decoding an empty array did not return a table")
   end
 
   r, x = pcall(decode, "[1e+2]")
@@ -404,6 +459,13 @@ else
     print ("1e+2 decoded incorrectly:", r[1])
   end
 
+  r, x = pcall(decode, "[1,2]")
+  if not r then
+    print ("decoding an array with two elements raises an error:", x)
+  elseif type(x) ~= 'table' or x[1] ~= 1 or x[2] ~= 2 then
+    print ("decoding an array with two elements did not return the expected table")
+  end
+
   inlocale(function ()
     local r, x = pcall(decode, "[0.5]")
     if not r then
@@ -417,7 +479,7 @@ else
 
   -- special tests for dkjson:
   if test_module == 'dkjson' then
-    x = dkdecode[=[ [{"x":0}] ]=]
+    x = decode[=[ [{"x":0}] ]=]
     local m = getmetatable(x)
     if not m or m.__jsontype ~= 'array' then
       print ("<metatable>.__jsontype ~= array")
@@ -427,17 +489,49 @@ else
       print ("<metatable>.__jsontype ~= object")
     end
     
-    local x,p,m = dkdecode" invalid "
+    local x,p,m = decode" invalid "
     if p ~= 2 or type(m) ~= 'string' or not m:find("at line 1, column 2$") then
-      print (("Invalid location: position=%d, message=%q"):format(p,m))
+      print (("Invalid location [1]: position=%d, message=%q"):format(p,m))
     end
-    local x,p,m = dkdecode" \n invalid "
+    local x,p,m = decode" \n invalid "
     if p ~= 4 or type(m) ~= 'string' or not m:find("at line 2, column 2$") then
-      print (("Invalid location: position=%d, message=%q"):format(p,m))
+      print (("Invalid location [2]: position=%d, message=%q"):format(p,m))
+    end
+    -- report the position of the unmatched opening character instead of the
+    -- position of the end of the string.
+    local x,p,m = decode"[   {\"a\":\"........\""
+    if type(m) ~= 'string' or not m:find("at line 1, column 5$") then
+      print (("Invalid location [3]: position=%d, message=%q"):format(p,m))
+    end
+    local x,p,m = decode"[   [\"a\",\"........\","
+    if type(m) ~= 'string' or not m:find("at line 1, column 5$") then
+      print (("Invalid location [4]: position=%d, message=%q"):format(p,m))
+    end
+    -- report the position of the last opened object
+    local x,p,m = decode"{\"x\":  {\"a\":\"........\""
+    if type(m) ~= 'string' or not m:find("at line 1, column 8$") then
+      print (("Invalid location [5]: position=%d, message=%q"):format(p,m))
+    end
+    -- report missing string for key
+    local x,p,m = decode"{  {\"a\":\"........\""
+    if type(m) ~= 'string' or not m:find("at line 1, column 4$") then
+      print (("Invalid location [6]: position=%d, message=%q"):format(p,m))
+    end
+    local x,p,m = decode"[-]"
+    if type(m) ~= 'string' or not m:find("at line 1, column 2$") then
+      print (("Invalid location for invalid char in array: position=%d, message=%q"):format(p,m))
+    end
+    local x,p,m = decode"{-}"
+    if type(m) ~= 'string' or not m:find("at line 1, column 2$") then
+      print (("Invalid location for invalid char in object: position=%d, message=%q"):format(p,m))
+    end
+    local x,p,m = decode"  \"...."
+    if type(m) ~= 'string' or not m:find("at line 1, column 3$") then
+      print (("Invalid location for unterminated string: position=%d, message=%q"):format(p,m))
     end
 
     do -- single line comments
-      local x, p, m  = dkdecode [[
+      local x, p, m  = decode [[
 {"test://" // comment // --?
    : [  // continues
    0]   //
@@ -449,7 +543,7 @@ else
     end
 
     do -- multi line comments
-      local x, p, m  = dkdecode [[
+      local x, p, m  = decode [[
 {"test:/*"/**//*
    hi! this is a comment
 */   : [/** / **/  0]
